Unit VESA;

Interface

Const
  g640x400x256       = $100;      { VESA SuperVGA video modes }
  g640x480x256       = $101;
  g800x600x16        = $102;
  g800x600x256       = $103;
  g1024x768x16       = $104;
  g1024x768x256      = $105;
  g1280x1024x16      = $106;
  g1280x1024x256     = $107;
  t80x60             = $108;
  t132x25            = $109;
  t132x43            = $10A;
  t132x50            = $10B;
  t132x60            = $10C;
  g320x200x32K       = $10D;      { VESA BIOS extensions 1.2 }
  g320x200x64K       = $10E;
  g320x200x16M       = $10F;
  g640x480x32K       = $110;
  g640x480x64K       = $111;
  g640x480x16M       = $112;
  g800x600x32K       = $113;
  g800x600x64K       = $114;
  g800x600x16M       = $115;
  g1024x768x32K      = $116;
  g1024x768x64K      = $117;
  g1024x768x16M      = $118;
  g1280x1024x32K     = $119;
  g1280x1024x64K     = $11A;
  g1280x1024x16M     = $11B;
  gS3_640x480x256    = $201;      { Values for S3 OEM video mode           }
  gS3_800x600x16     = $202;
  gS3_800x600x256    = $203;
  gS3_1024x768x16    = $204;
  gS3_1024x768x256   = $205;
  gS3_1280x960x16    = $206;
  gS3_1280x1024x16   = $208;
  gS3_640x480x32K    = $301;

  VideoHardwareState = 1;         { State constants for saving/restoring   }
  VideoBIOSDataState = 2;         {  the video state                       }
  VideoDACState      = 4;
  VideoSuperVGAState = 8;

Type
  WindowName = (WindowA,WindowB);
  VESAInfoRec = Record
    Signature          : Array[1..4] Of Byte;   { Should be "VESA"        }
    Version            : Word;    { VESA version number                   }
    OEMPtr             : Pointer; { Pointer to OEM name ("761295520" for  }
                                  {  ATI) }
    Capabilities       : LongInt; { Capabilities                          }
    ModeListPtr        : Pointer; { Pointer to list of supported OEM video}
                                  {  modes (list of words terminated with }
                                  {  FFFFh).                              }
    Reserved           : Array[1..238] Of Byte;
  End;
  VESAModeInfoRec = Record
    ModeAttr           : Word;    { Mode attributes                       }
    WindowAAttr        : Byte;    { Window A attributes                   }
    WindowBAttr        : Byte;    { Window B attributes                   }
    WindowGranularity  : Word;    { Smallest Kb increment by which window }
                                  {  can be moved                         }
    WindowSize         : Word;    { Window size in K                      }
    WinAStart          : Word;    { Start segment of window A             }
    WinBStart          : Word;    { Start segment of window B             }
    PosFunction        : Pointer; { FAR window positioning function       }
    BytesPerLine       : Word;    { Bytes per scan line                   }

    { Remainder is optional for VESA modes in v1.0/1.1, needed for OEM    }
    { modes                                                               }

    Width              : Word;    { Width in pixels                       }
    Height             : Word;    { Height in pixels                      }
    CellWidth          : Byte;    { Width of character cell in pixels     }
    CellHeight         : Byte;    { Height of character cell in pixels    }
    Planes             : Byte;    { Number of memory planes               }
    BPP                : Byte;    { Number of bits per pixel              }
    Banks              : Byte;    { Number of banks                       }
    MemModelType       : Byte;    { Memory model type                     }
    BankSize           : Byte;    { Size of bank in K                     }
    NumImgPages        : Byte;    { Number of image pages                 }
    Reserved1          : Byte;

    { VBE v1.2+                                                           }

    RMaskSize          : Byte;    { Red mask size                         }
    RFieldPos          : Byte;    { Red field size                        }
    GMaskSize          : Byte;    { Green mask size                       }
    GFieldPos          : Byte;    { Green field size                      }
    BMaskSize          : Byte;    { Blue mask size                        }
    BFieldPos          : Byte;    { Blue field size                       }
    ResMaskSize        : Byte;    { Reserved mask size                    }
    ResMaskPos         : Byte;    { Reserved mask position                }
    DirColModeInfo     : Byte;    { Direct color mode info                }
    Reserved2          : Array[1..216] Of Byte;
  End;

  { Notes -------------------------------------------------------------- }
  {                                                                      }
  { ModeAttr - mode attributes                                           }
  {                                                                      }
  {   bit 0: mode supported                                              }
  {   bit 1: optional information available                              }
  {   bit 2: BIOS output supported                                       }
  {   bit 3: set if color, clear if monochrome                           }
  {   bit 4: set if graphics mode, clear if text mode                    }
  {                                                                      }
  { WindowAAttr/WindowBAttr - window A/B attributes                      }
  {                                                                      }
  {   bit 0: exists                                                      }
  {   bit 1: readable                                                    }
  {   bit 2: writable                                                    }
  {   bits 3-7: reserved                                                 }
  {                                                                      }
  { Values for memory model type                                         }
  {                                                                      }
  {   00h ......... text                                                 }
  {   01h ......... CGA graphics                                         }
  {   02h ......... HGC graphics                                         }
  {   03h ......... 16-color (EGA) graphics                              }
  {   04h ......... packed pixel graphics                                }
  {   05h ......... "sequ 256" (non-chain 4) graphics                    }
  {   06h ......... direct color (HiColor, 24-bit color)                 }
  {   07h ......... YUV (luminance-chrominance, also called YIQ)         }
  {   08h-0Fh ..... reserved for VESA                                    }
  {   10h-FFh ..... OEM memory models                                    }
  {                                                                      }
  { -------------------------------------------------------------------- }

Function GetSuperVGAInfo(Var InfoRec: VESAInfoRec): Boolean;
Function GetSuperVGAModeInfo(Mode: Word; Var ModeRec: VESAModeInfoRec): Boolean;
Function SetSuperVGAMode(Mode: Word): Boolean;
Function GetCurrentSuperVGAMode(Var Mode: Word): Boolean;
Function GetSuperVGAStateSize(Flags: Word; Var Blocks: Word): Boolean;
Function SaveSuperVGAState(Flags: Word; Var Buffer: Pointer): Boolean;
Function RestoreSuperVGAState(Flags: Word; Var Buffer: Pointer): Boolean;
Function SetMemoryWindow(WindowAddr: Word): Boolean;
Function GetMemoryWindow(Var WindowAddr: Word; Var WindowNum: WindowName): Boolean;
Function SetScanLineLength(LengthInPixels: Word): Boolean;
Function GetScanLineLength(Var BytesPerLine: Word; Var PixelsPerLine: Word; Var MaxLines: Word): Boolean;
Function SetDisplayStart(Left,Top: Word): Boolean;
Function GetDisplayStart(Var Left: Word; Var Top: Word): Boolean;

Implementation

Uses Dos;

Var Regs: Registers;

Function GetSuperVGAInfo(Var InfoRec: VESAInfoRec): Boolean;
Begin
  Regs.AX := $4F00;
  Regs.ES := Seg(InfoRec);
  Regs.DI := Ofs(InfoRec);
  Intr($10,Regs);
  GetSuperVGAInfo := (Regs.AX = $4F);
End; { GetSuperVGAInfo }


Function GetSuperVGAModeInfo(Mode: Word; Var ModeRec: VESAModeInfoRec): Boolean;
Begin
  Regs.AX := $4F01;
  Regs.CX := Mode;
  Regs.ES := Seg(ModeRec);
  Regs.DI := Ofs(ModeRec);
  Intr($10,Regs);
  GetSuperVGAModeInfo := (Regs.AX = $4F);
End; { GetSuperVGAModeInfo }


Function SetSuperVGAMode(Mode: Word): Boolean;
Begin
  Regs.AX := $4F02;
  Regs.BX := Mode;
  Intr($10,Regs);
  SetSuperVGAMode := (Regs.AX = $4F);
End; { SetSuperVGAMode }


Function GetCurrentSuperVGAMode(Var Mode: Word): Boolean;
Begin
  Regs.AX := $4F03;
  Intr($10,Regs);
  If Regs.AX = $4F Then
  Begin
    Mode := Regs.BX;
    GetCurrentSuperVGAMode := True;
  End
  Else GetCurrentSuperVGAMode := False;
End; { GetCurrentSuperVGAMode }


Function GetSuperVGAStateSize(Flags: Word; Var Blocks: Word): Boolean;
Begin
  Regs.AX := $4F04;
  Regs.CX := Flags;
  Regs.DL := 0;
  Intr($10,Regs);
  If Regs.AX = $4F Then
  Begin
    Blocks               := Regs.BX;
    GetSuperVGAStateSize := True;
  End
  Else GetSuperVGAStateSize := False;
End; { GetSuperVGAStateSize }


Function SaveSuperVGAState(Flags: Word; Var Buffer: Pointer): Boolean;
Begin
  Regs.AX := $4F04;
  Regs.CX := Flags;
  Regs.DL := 1;
  Regs.ES := Seg(Buffer^);
  Regs.BX := Ofs(Buffer^);
  Intr($10,Regs);
  SaveSuperVGAState := (Regs.AX = $4F);
End; { SaveSuperVGAState }


Function RestoreSuperVGAState(Flags: Word; Var Buffer: Pointer): Boolean;
Begin
  Regs.AX := $4F04;
  Regs.CX := Flags;
  Regs.DL := 2;
  Regs.ES := Seg(Buffer^);
  Regs.BX := Ofs(Buffer^);
  Intr($10,Regs);
  RestoreSuperVGAState := (Regs.AX = $4F);
End; { RestoreSuperVGAState }


Function SetMemoryWindow(WindowAddr: Word): Boolean;
Begin
  Regs.AX := $4F05;
  Regs.BH := 0;
  Regs.DX := WindowAddr;
  Intr($10,Regs);
  SetMemoryWindow := (Regs.AX = $4F);
End; { SetMemoryWindow }


Function GetMemoryWindow(Var WindowAddr: Word; Var WindowNum: WindowName): Boolean;
Begin
  Regs.AX := $4F05;
  Regs.BH := 1;
  Intr($10,Regs);
  If Regs.AX = $4F Then
  Begin
    WindowAddr      := Regs.DX;
    WindowNum       := WindowName(Regs.BL);
    GetMemoryWindow := True;
  End
  Else GetMemoryWindow := False;
End; { GetMemoryWindow }


Function SetScanLineLength(LengthInPixels: Word): Boolean;
Begin
  Regs.AX := $4F06;
  Regs.BL := 0;
  Regs.CX := LengthInPixels;
  Intr($10,Regs);
  SetScanLineLength := (Regs.AX = $4F);
End; { SetScanLineLength }


Function GetScanLineLength(Var BytesPerLine: Word; Var PixelsPerLine: Word; Var MaxLines: Word): Boolean;
Begin
  Regs.AX := $4F06;
  Regs.BL := 1;
  Intr($10,Regs);
  If Regs.AX = $4F Then
  Begin
    BytesPerLine      := Regs.BX;
    PixelsPerLine     := Regs.CX;
    MaxLines          := Regs.DX;
    GetScanLineLength := True;
  End
  Else GetScanLineLength := False;
End; { GetScanLineLength }


Function SetDisplayStart(Left,Top: Word): Boolean;
Begin
  Regs.AX := $4F07;
  Regs.BH := 0;
  Regs.BL := 0;
  Regs.CX := Left;
  Regs.DX := Top;
  Intr($10,Regs);
  SetDisplayStart := (Regs.AX = $4F);
End; { SetDisplayStart }


Function GetDisplayStart(Var Left: Word; Var Top: Word): Boolean;
Begin
  Regs.AX := $4F07;
  Regs.BH := 0;
  Regs.BL := 1;
  Intr($10,Regs);
  If Regs.AX = $4F Then
  Begin
    Left            := Regs.CX;
    Top             := Regs.DX;
    GetDisplayStart := True;
  End
  Else GetDisplayStart := False;
End; { GetDisplayStart }

End.

// ----------------------------------------------------------------------
// PCAE - PC Atari Emulator - Atari 2600 Emulator for MS-DOS
// Copyright (C) 2000 John Dullea
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
// ----------------------------------------------------------------------

